:- module(hs_parser,_,[assertions]).

:- use_module(library(lists)).
:- doc(title, "Parser").
:- doc(author, "Christian Giménez").
:- doc(copyright, "
hs_parser.pl
Copyright (C) 2012  Giménez, Christian N.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
Domingo 09 De Septiembre Del 2012    
").
:- pred parse_http(+ListStr, -File) # "True happens when @var{ListStr} is a correct GET text and the @var{File} is the File that is requesting.".
parse_http(ListStr,File) :- 
	show_liststr(ListStr),
	verify_get(ListStr),
	obtain_file(ListStr, File).

:- pred verify_get(+ListStr) # "Return true if the GET text is correct.".
verify_get([X|_]) :-
	append("GET /", _, X),
	append("HTTP/1.1", [13,10], Z),
	append(_, Z, X).

:- pred obtain_file(+ListStr, -File) # "Retrieve the file relative path that the client is requesting
	      this is true when @var{File} is the file that the first sentence of the @var{ListStr}
	      is pointing in the @tt{GET ... HTTP/1.1} GET sentence.".
obtain_file([X|_], File) :-
	append(" HTTP/1.1", [13,10], Z),
	append("GET ", Y, X),	
	append(File, Z, Y).


:- doc(section, "Show for Debugging  Purpose").
:- pred show_liststr(ListStr) # "Show at terminal output what it is going to be parsed.".
show_liststr([]) :-
	nl,
	display(done),
	nl.

show_liststr([X|L]) :-
	nl,
	display_string(X),
	show_liststr(L).

