:- module(hs_server, [start_server/0], [assertions]).

:- doc(title, "Server").
:- doc(author, "Christian Giménez").
:- doc(copyright, "
hs_server.pl
Copyright (C) 2012  Giménez, Christian N.

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
Domingo 09 De Septiembre Del 2012    
").

% --------------------

:- use_module(library(sockets)).
:- use_module(library(lists)).
:- use_module(library(iso_byte_char)).
:- use_module(library(write)).
:- use_module(hs_config).
:- use_module(hs_parser).
:- use_module(hs_file).

:- pred start_server 
	# "Start the server using information from @tt{hs_config} module.
This is true if starting the server and parsing information was performed accordingly.".
start_server :-
	port(Port),
	bind_socket(Port, 1, Socket),
	accept_next_client(Socket).

:- pred accept_next_client(+Socket) # "Given a @var{Socket} it becames true if retrieving and parsing is fine.".
accept_next_client(Socket) :-
	socket_accept(Socket, Stream),
	process_client(Stream),
	close(Stream),
	accept_next_client(Socket).

process_client(Stream) :-
	recieve_request(Stream,L),
	parse_http(L,File),
	send_info(Stream, File).

recieve_request(Stream, L) :-
	socket_recv(Stream, X),
	% display_string(X),nl,
	% display(X),
	socket_recv(Stream, Y),	
	% display_string(Y),nl,
	% display(Y),
	decide_recieve_again(Stream,X,Y,L).

decide_recieve_again(_, X, [13,10], [X,[13,10]]) :- 
	append(_,[13,10], X),!.

decide_recieve_again(Stream, X, Y, [X,Y|L]) :- 
	recieve_request(Stream, L).

:- pred send_info(+Stream, +File) # "Is true if the @var{File} is opened and its content sended throught the given @var{Stream}.".
send_info(Stream, File) :-
	open_file(File,Str), 
	% take_and_send(Stream),
	socket_send(Stream,Str).
% close_file.

:- pred take_and_send(+Stream) # "Take a byte and send it!".
% take_and_send(_) :-
% 	next_byte(end_of_file),!.

take_and_send(Stream) :-
	next_byte(B),
	socket_send(Stream, B),!.
	%take_and_send(Stream).
